#include <algorithm>
#include <array>
#include <cassert>
#include <iostream>
#include <unordered_map>
#include <vector>

#define all(x) begin(x), end(x)
#define sz(x) (int((x).size()))
#define rep(i, a, b) for (int i = (a); i < (b); ++i)

using vi = std::vector<int>;
using vvi = std::vector<vi>;

// https://github.com/kth-competitive-programming/kactl/blob/main/content/graph/EdmondsKarp.h
// licensed under CC0  :)
int edmondsKarp(std::vector<std::unordered_map<int, int>> &graph, int source, int sink) {
    assert(source != sink);
    int flow = 0;
    vi par(sz(graph)), q = par;

    for (;;) {
        fill(all(par), -1);
        par[source] = 0;
        int ptr = 1;
        q[0] = source;

        rep(i,0,ptr) {
            int x = q[i];
            for (auto e : graph[x]) {
                if (par[e.first] == -1 && e.second > 0) {
                    par[e.first] = x;
                    q[ptr++] = e.first;
                    if (e.first == sink) goto out;
                }
            }
        }
        return flow;
out:
        int inc = 1e9;
        for (int y = sink; y != source; y = par[y])
            inc = std::min(inc, graph[par[y]][y]);

        flow += inc;
        for (int y = sink; y != source; y = par[y]) {
            int p = par[y];
            if ((graph[p][y] -= inc) <= 0) graph[p].erase(y);
            graph[y][p] += inc;
        }
    }
}

bool permitsPerfectMatching(const vvi &g, int sizeRight) {
    int sizeLeft = g.size();
    int source = sizeLeft + sizeRight;
    int sink = source + 1;

    std::vector<std::unordered_map<int, int>> network(sink + 1);
    for (int i = 0; i < sizeLeft; i++) {
        network[source].insert({ i, 1 });
        for (int j : g[i]) network[i].insert({ j + sizeLeft, 1 });
    }
    for (int j = 0; j < sizeRight; j++) {
        network[j + sizeLeft].insert({ sink, 1 });
    }

    return edmondsKarp(network, source, sink) == sizeRight;
}

struct card {
    char letter, digit;
};

void print(const vi &v) {
    assert(!v.empty());
    std::cout << v.size() << '\n';
    for (int i = 0; i < sz(v); i++) {
        if (i) std::cout << ' ';
        std::cout << v[i] + 1;
    }
    std::cout << std::endl;
}

int main() {
    int n, k;  // no. cards in deck and on table, respectively
    std::cin >> n >> k;

    std::vector<card> deck(n);
    for (card &c : deck) std::cin >> c.letter >> c.digit;

    std::vector<char> table;
    for (int j = 0; j < k; j++) {
        char c;
        std::cin >> c;
        table.push_back(c);
    }

    char D, L;           // claim: "Each card on the table with a {D} on
    std::cin >> D >> L;  //         one side has an {L} on the other side."

    // the claim is essentially  D => L  ≡  ¬D ∨ L
    // so the "illegal" cards are those with  D ∧ ¬L
    vi legal, illegal;
    for (int i = 0; i < n; i++) {
        auto [l, d] = deck[i];
        (l != L && d == D ? illegal : legal).push_back(i);
    }

    if (illegal.empty()) {
        std::cout << "true" << std::endl;
        return 0;
    }

    // If, considering only legal cards, we cannot find a matching between cards
    // in the deck and on the table, then the table must contain an illegal card,
    // so the claim is certainly false.
    {
        vvi g(deck.size());
        for (int i : legal) {
            auto [l, d] = deck[i];
            for (int j = 0; j < k; j++)
                if (l == table[j] || d == table[j])
                    g[i].push_back(j);
        }

        if (!permitsPerfectMatching(g, table.size())) {
            std::cout << "false" << std::endl;
            return 0;
        }
    }

    // Now, the claim may be either true or false: a legal matching exists, but
    // any legal card on the table could actually be an illegal one, as long as
    // the upright side is the same. Hence, we now determine which cards we need
    // to flip to (dis)prove the claim.

    std::array<bool, 26> checkLetter;
    checkLetter.fill(false);
    for (int i : illegal) checkLetter[deck[i].letter - 'A'] = true;
    assert(!checkLetter[L - 'A']);

    vi flip;
    for (int j = 0; j < k; j++) {
        char c = table[j];

        // Since we know the deck contains illegal cards, we must flip all cards
        // with a {D}.
        if (c == D) flip.push_back(j);

        // If a table card shows a letter `l` and there exists an illegal card
        // with that letter, we must flip it. Note that we never have to flip
        // cards with `l = L`.
        if (c >= 'A' && checkLetter[c - 'A']) flip.push_back(j);
    }

    if (flip.empty())
        // The table contains no illegal cards (since none of the visible sides
        // match). Therefore, all existing matchings are legal.
        std::cout << "true" << std::endl;
    else
        print(flip);
}
