#!/usr/bin/env python3
# @EXPECTED_RESULTS@: ACCEPTED, TIME_LIMIT_EXCEEDED

""" Top-down recursive solution with memoization (using Python's functools.lru_cache).
    Time O(n k log n), can be sped up to O(n k + max D) where D is the maximum date,
    much like in ragnar's bottom-up dynamic programming solution.

    With '-c', instead computes the *number* of optimal solutions.
    Run this as, e.g.,

        find . -name "*.in" -print0 | xargs -0 -I {} sh -c 'python3 submissions/accepted/th-rec.py -c -s < {}'

    With '-s', if the solution is unique, computes statistics about which passes are needed
"""

from bisect import bisect_left as floor_idx
from functools import lru_cache
from collections import Counter
import sys

n, k = map(int, input().split())
trips = tuple(tuple(map(int, input().split())) for _ in range(n))
dates = tuple(t[0] for t in trips)
fares = tuple(t[1] for t in trips)
passes = tuple(tuple(map(int, input().split())) for _ in range(k))

statistics = "-s" in sys.argv and k <= 1000

sys.setrecursionlimit(100000)
@lru_cache(maxsize=100000)
def cost(idx):
    """Returns 
    - the cheapest total cost for rides up to and including idx
    - number of solutions attaining that cost
    - if the solution is unique: counter of which passes are used
    """
    if idx < 0:
        return (0, 1, Counter()) if statistics else (0, 1, None)
    # case: pay normal fare for day idx
    prev_cost, num_solutions, counter = cost(idx -1)
    ctr = counter + Counter({0:1}) if statistics else None
    choices = [(prev_cost + fares[idx], num_solutions, None, ctr)]
    # other cases: buy a pass
    for pidx, (days, rides, price) in enumerate(passes):
        earliest_free_idx = max(
            idx - rides + 1, floor_idx(dates, dates[idx] - days + 1)
        )
        prev_cost, num_solutions, counter = cost(earliest_free_idx - 1)
        ctr = counter + Counter({pidx+1:1}) if statistics else None
        choices.append((prev_cost + price, num_solutions, pidx, ctr))
    # find min-cost choice, also compute number of such choices for statistics
    min_cost = min(choices, key=lambda t:t[0])[0]
    min_choices = list(c for c in choices if c[0] == min_cost)
    return min_cost, len(min_choices), min_choices[0][3]


opt, num_opt, ctr = cost(n-1)
if "-c" in sys.argv or "-s" in sys.argv:
    print(f"{n:>4} {k:>4} {opt:>8}", end=' ')
else:
    print(opt, end='')
if "-c" in sys.argv:
    print(num_opt, end=' ')
if "-s" in sys.argv:
    if num_opt == 1:
        print(ctr, end=' ')
print()
