#include <iostream>
#include <vector>

struct edge {
    int i, s, t;
    edge() : i(-1), s(-1), t(-1) {}
    edge(int i, int s, int t) : i(i), s(s), t(t) {}
    bool exists() const { return i != -1; }
};

struct edge_dual {
    int t;          // index of target vertex (in dual, i.e., edge in primal)
    bool blinkers;  // whether blinkers must be turned on to take this edge
};

struct state {
    int v;          // index of vertex (in dual, i.e., edge in primal)
    int length;     // .. of path
    int blinkers;   // number of times blinkers were turned on
    state append(const edge_dual e) const { return { e.t, length + 1, blinkers + e.blinkers }; }
};

using ve   = std::vector<edge>;
using ved  = std::vector<edge_dual>;
using vve  = std::vector<ve>;
using vved = std::vector<ved>;

vve primal;
ve edges;
vved layeredDual;

int main() {
    int n, k;
    std::cin >> n >> k;

    const int start = 0;
    const int dest = n-1;

    // edge case
    if (start == dest) {
        std::cout << 0 << std::endl;
        return 0;
    }

    // read primal
    int m = 0;
    primal = vve(n, ve(4));
    for (int u = 0; u < n; u++) {
        for (int d = 0; d < 4; d++) {  // north, east, south, west (clockwise)
            int v;
            std::cin >> v;
            if (v) {
                const edge e { m++, u, v-1 };  // convert to 0-based
                primal[u][d] = e;
                edges.push_back(e);
            }
        }
    }

    // construct dual, i.e., (directed) line graph
    // this is most likely possible with less code, but I like the explicitness for now
    //   layer 0: blinkers off
    //   layer 1: left blinkers on
    //   layer 2: right blinkers on
    layeredDual = vved(3*m);
    for (const edge e : edges) {
        int d = 0;
        while (primal[e.t][d].t != e.s) d = (d+1) % 4;

        // left turn
        d = (d+1) % 4;
        const edge left = primal[e.t][d];
        if (left.exists()) {
            layeredDual[e.i      ].push_back({ left.i + m, true  });
            layeredDual[e.i +   m].push_back({ left.i + m, false });
            layeredDual[e.i + 2*m].push_back({ left.i + m, true  });
        }

        // straight ahead
        d = (d+1) % 4;
        const edge straight = primal[e.t][d];
        if (straight.exists()) {
            layeredDual[e.i      ].push_back({ straight.i, false });
            layeredDual[e.i +   m].push_back({ straight.i, false });
            layeredDual[e.i + 2*m].push_back({ straight.i, false });
        }

        // right turn
        d = (d+1) % 4;
        const edge right = primal[e.t][d];
        if (right.exists()) {
            layeredDual[e.i      ].push_back({ right.i + 2*m, true  });
            layeredDual[e.i +   m].push_back({ right.i + 2*m, true  });
            layeredDual[e.i + 2*m].push_back({ right.i + 2*m, false });
        }
    }

    // prepare queue
    std::vector<state> queue;
    std::vector<std::vector<bool>> visited(k+1, std::vector<bool>(layeredDual.size()));
    for (int d = 0; d < 4; d++) {
        const edge e = primal[start][d];
        if (e.exists()) {
            queue.push_back({ e.i, 1, 0 });
            visited[0][e.i] = true;
        }
    }

    // bfs
    while (!queue.empty()) {
        // find and delete minimum in linear time (poor man's priority queue)
        auto currIt = queue.begin();
        for (auto it = currIt + 1; it != queue.end(); ++it)
            if (it->length < currIt->length)
                currIt = it;

        const auto curr = *currIt;
        queue.erase(currIt);

        /*
        std::cerr << "state (" << curr.v << ", " << curr.length << ", " << curr.blinkers << ") : "
                  << "edge " << curr.v % m << " from " << edges[curr.v % m].s + 1 << " to " << edges[curr.v % m].t + 1 << std::endl;
        */

        if (edges[curr.v % m].t == dest) {
            std::cout << curr.length << std::endl;
            return 0;
        }

        for (const edge_dual e : layeredDual[curr.v]) {
            const state next = curr.append(e);
            if (next.blinkers > k || visited[next.blinkers][next.v]) continue;
            queue.push_back(next);
            visited[next.blinkers][next.v] = true;
        }
    }

    std::cout << "impossible" << std::endl;
}
