#include <algorithm>
#include <array>
#include <cassert>
#include <chrono>
#include <iostream>
#include <numeric>
#include <random>
#include <vector>

#define all(x) begin(x), end(x)
#define sz(x) (int((x).size()))
#define rep(i, a, b) for (int i = (a); i < (b); ++i)
#define now() (std::chrono::steady_clock::now())

using vi = std::vector<int>;
using vvi = std::vector<vi>;

std::random_device randomDevice;
std::mt19937 randomGenerator(randomDevice());

int getRandom(const vi &v) {
    std::uniform_int_distribution uni(0, sz(v) - 1);
    return v[uni(randomGenerator)];
}

int sinceMs(std::chrono::steady_clock::time_point start) {
    return std::chrono::duration_cast<std::chrono::milliseconds>(now() - start).count();
}

// Error: we use a non-deterministic approach.
// Note left/right are swapped compared to the correct submissions.
bool permitsPerfectMatching(const vvi &g, int rightSize) {
    int leftSize = g.size();
    if (leftSize > rightSize) return false;

    // the list of left nodes, in (random) processing order
    vi left(leftSize);
    std::iota(all(left), 0);

    // for each right node, whether it's already used in the current matching
    std::vector<bool> rightUsed(rightSize);

    // reusable buffer of (right) nodes
    vi buffer;
    buffer.reserve(rightSize);

    // greedily try to construct many different matchings, and return true if we find a valid one
    for (const auto timeStart = now(); sinceMs(timeStart) < 900; ) {
        std::shuffle(all(left), randomGenerator);
        std::fill(all(rightUsed), false);
        for (int j : left) {
            buffer.clear();
            for (int i : g[j])
                if (!rightUsed[i])
                    buffer.push_back(i);
            if (buffer.empty()) goto next;
            rightUsed[getRandom(buffer)] = true;
        }
        return true;  // we were able to match each left node with one on the right
next:   ;
    }
    return false;  // we couldn't find a matching, so assume there doesn't exist one
}

struct card {
    char letter, digit;
};

void print(const vi &v) {
    assert(!v.empty());
    std::cout << v.size() << '\n';
    for (int i = 0; i < sz(v); i++) {
        if (i) std::cout << ' ';
        std::cout << v[i] + 1;
    }
    std::cout << std::endl;
}

int main() {
    int n, k;  // no. cards in deck and on table, respectively
    std::cin >> n >> k;

    std::vector<card> deck(n);
    for (card &c : deck) std::cin >> c.letter >> c.digit;

    std::vector<char> table;
    for (int j = 0; j < k; j++) {
        char c;
        std::cin >> c;
        table.push_back(c);
    }

    char D, L;           // claim: "Each card on the table with a {D} on
    std::cin >> D >> L;  //         one side has an {L} on the other side."

    // the claim is essentially  D => L  ≡  ¬D ∨ L
    // so the "illegal" cards are those with  D ∧ ¬L
    vi legal, illegal;
    for (int i = 0; i < n; i++) {
        auto [l, d] = deck[i];
        (l != L && d == D ? illegal : legal).push_back(i);
    }

    if (illegal.empty()) {
        std::cout << "true" << std::endl;
        return 0;
    }

    // If, considering only legal cards, we cannot find a matching between cards
    // in the deck and on the table, then the table must contain an illegal card,
    // so the claim is certainly false.
    {
        vvi g(table.size());
        for (int j = 0; j < k; j++) {
            char c = table[j];
            for (int i : legal) {
                auto [l, d] = deck[i];
                if (l == c || d == c) g[j].push_back(i);
            }
        }

        if (!permitsPerfectMatching(g, deck.size())) {
            std::cout << "false" << std::endl;
            return 0;
        }
    }

    // Now, the claim may be either true or false: a legal matching exists, but
    // any legal card on the table could actually be an illegal one, as long as
    // the upright side is the same. Hence, we now determine which cards we need
    // to flip to (dis)prove the claim.

    std::array<bool, 26> checkLetter;
    checkLetter.fill(false);
    for (int i : illegal) checkLetter[deck[i].letter - 'A'] = true;
    assert(!checkLetter[L - 'A']);

    vi flip;
    for (int j = 0; j < k; j++) {
        char c = table[j];

        // Since we know the deck contains illegal cards, we must flip all cards
        // with a {D}.
        if (c == D) flip.push_back(j);

        // If a table card shows a letter `l` and there exists an illegal card
        // with that letter, we must flip it. Note that we never have to flip
        // cards with `l = L`.
        if (c >= 'A' && checkLetter[c - 'A']) flip.push_back(j);
    }

    if (flip.empty())
        // The table contains no illegal cards (since none of the visible sides
        // match). Therefore, all existing matchings are legal.
        std::cout << "true" << std::endl;
    else
        print(flip);
}
